@@ -159,6 +159,13 @@ implements ArrayAccess, Countable, RecursiveIterator, Serializable
      */
     protected $_transferEncoding = self::DEFAULT_ENCODING;
 
+    /**
+     * Flag to detect if a message failed to send at least once.
+     *
+     * @var boolean
+     */
+    protected $_failed = false;
+
     /**
      * Constructor.
      */
@@ -1436,6 +1443,67 @@ implements ArrayAccess, Countable, RecursiveIterator, Serializable
         }
     }
 
+    /**
+     * Determines if this MIME part is an attachment for display purposes.
+     *
+     * @since Horde_Mime 2.10.0
+     *
+     * @return boolean  True if this part should be considered an attachment.
+     */
+    public function isAttachment()
+    {
+        $type = $this->getType();
+
+        switch ($type) {
+        case 'application/ms-tnef':
+        case 'application/pgp-keys':
+        case 'application/vnd.ms-tnef':
+            return false;
+        }
+
+        if ($this->parent) {
+            switch ($this->parent->getType()) {
+            case 'multipart/encrypted':
+                switch ($type) {
+                case 'application/octet-stream':
+                    return false;
+                }
+                break;
+
+            case 'multipart/signed':
+                switch ($type) {
+                case 'application/pgp-signature':
+                case 'application/pkcs7-signature':
+                case 'application/x-pkcs7-signature':
+                    return false;
+                }
+                break;
+            }
+        }
+
+        switch ($this->getDisposition()) {
+        case 'attachment':
+            return true;
+        }
+
+        switch ($this->getPrimaryType()) {
+        case 'application':
+            if (strlen($this->getName())) {
+                return true;
+            }
+            break;
+
+        case 'audio':
+        case 'video':
+            return true;
+
+        case 'multipart':
+            return false;
+        }
+
+        return false;
+    }
+
     /**
      * Set a piece of metadata on this object.
      *
@@ -1557,6 +1625,19 @@ implements ArrayAccess, Countable, RecursiveIterator, Serializable
                 'canonical' => $canonical,
                 'charset' => $this->getHeaderCharset()
             )), $msg);
+        } catch (InvalidArgumentException $e) {
+            // Try to rebuild the part in case it was due to
+            // an invalid line length in a rfc822/message attachment.
+            if ($this->_failed) {
+                throw $e;
+            }
+            $this->_failed = true;
+            $this->_sanityCheckRfc822Attachments();
+            try {
+                $this->send($email, $headers, $mailer, $opts);
+            } catch (Horde_Mail_Exception $e) {
+                throw new Horde_Mime_Exception($e);
+            }
         } catch (Horde_Mail_Exception $e) {
             throw new Horde_Mime_Exception($e);
         }
@@ -1640,7 +1721,6 @@ implements ArrayAccess, Countable, RecursiveIterator, Serializable
         }
     }
 
-
     /**
      * Write data to a stream.
      *
@@ -2113,6 +2193,39 @@ implements ArrayAccess, Countable, RecursiveIterator, Serializable
         return $out;
     }
 
+    /**
+     * Re-enocdes message/rfc822 parts in case there was e.g., some broken
+     * line length in the headers of the message in the part. Since we shouldn't
+     * alter the original message in any way, we simply reset cause the part to
+     * be encoded as base64 and sent as a application/octet part.
+     */
+    protected function _sanityCheckRfc822Attachments()
+    {
+        if ($this->getType() == 'message/rfc822') {
+            $this->_reEncodeMessageAttachment($this);
+            return;
+        }
+        foreach ($this->getParts() as $part) {
+            if ($part->getType() == 'message/rfc822') {
+                $this->_reEncodeMessageAttachment($part);
+            }
+        }
+        return;
+    }
+
+    /**
+     * Rebuilds $part and forces it to be a base64 encoded
+     * application/octet-stream part.
+     *
+     * @param  Horde_Mime_Part $part   The MIME part.
+     */
+    protected function _reEncodeMessageAttachment(Horde_Mime_Part $part)
+    {
+        $new_part = Horde_Mime_Part::parseMessage($part->getContents());
+        $part->setContents($new_part->getContents(array('stream' => true)), array('encoding' => self::ENCODE_BINARY));
+        $part->setTransferEncoding('base64', array('send' => true));
+    }
+
     /* ArrayAccess methods. */
 
     /**
